import AwsLogStream

from AwsProcessor import AwsProcessor
from stdplusAwsHelpers.AwsConnectionFactory import AwsConnectionFactory
from CommandArgumentParser import CommandArgumentParser

from pprint import pprint

class AwsLogGroup(AwsProcessor):
    def __init__(self,stackResource,parent):
        """Construct an AwsLogGroup command processor"""
        AwsProcessor.__init__(self,parent.raw_prompt + "/logGroup:" + stackResource.logical_id,parent)
        self.stackResource = stackResource
        self.logStreamNamePrefix = None
        self.orderBy = 'LogStreamName'
        self.descending = False
        self.do_refresh('')

    def do_refresh(self,args):
        """Refresh the view of the log group"""
        # prints all the groups: pprint(AwsConnectionFactory.getLogClient().describe_log_groups())

        response = AwsConnectionFactory.getLogClient().describe_log_groups(logGroupNamePrefix=self.stackResource.physical_resource_id)
        if not 'logGroups' in response:
            raise Exception("Expected log group description to have logGroups entry. Got {}".format(response))

        # pprint(response)
        descriptions = [x for x in response['logGroups'] if x['logGroupName'] == self.stackResource.physical_resource_id]
        if not descriptions:
            raise Exception("Could not find log group {} in list {}".format(self.stackResource.physical_resource_id,response['logGroups']))

        self.description = descriptions[0]

        self.logStreams = self.loadLogStreams()
        print "== logStream"
        maxIndex = "{}".format(len(self.logStreams)+1)
        print "maxIndex:{}".format(maxIndex)
        frm = "  {{0:{}d}}: {{1}}".format(len(maxIndex))
        print frm

        index = 0
        for logStream in self.logStreams:
            print frm.format(index,logStream['logStreamName'])
            index += 1

    def loadLogStreams(self):
        response = AwsConnectionFactory.getLogClient().describe_log_streams(logGroupName=self.stackResource.physical_resource_id,
                                                                                           #logStreamNamePrefix=self.logStreamNamePrefix,
                                                                                           orderBy=self.orderBy,
                                                                                           descending=self.descending)
        
        logStreams = response['logStreams']
        for logStream in logStreams:
            logStream['logGroupName']=self.stackResource.physical_resource_id
            
        return logStreams

    def do_logStream(self,args):
        """Go to the specified log stream. logStream -h for detailed help"""
        parser = CommandArgumentParser("logStream")
        parser.add_argument(dest='logStream',help='logStream index.');
        args = vars(parser.parse_args(args))

        print "loading log stream {}".format(args['logStream'])
        index = int(args['logStream'])
        logStream = self.logStreams[index]

        print "logStream:{}".format(logStream)
        self.childLoop(AwsLogStream.AwsLogStream(logStream,self))
        #self.stackResource(logStream.stack_name,logStream.logical_id)
        



from AwsProcessor import AwsProcessor
from stdplusAwsHelpers.AwsConnectionFactory import AwsConnectionFactory

from pprint import pprint

class AwsRole(AwsProcessor):
    def __init__(self,stackResource,parent):
        """Construct an AwsRole command processor"""
        AwsProcessor.__init__(self,parent.raw_prompt + "/role:" + stackResource.logical_id,parent)
        self.stackResource = stackResource
        self.do_refresh('')

    def do_refresh(self,args):
        """Refresh the view of the log group"""
        print "stackResource: {}".format(self.stackResource)
        
        self.roleDetails = AwsConnectionFactory.getIamClient().get_role(RoleName=self.stackResource.physical_resource_id)
        print "== role details =="
        pprint(self.roleDetails)

        self.rolePolicies = self.loadRolePolicies()
        print "== attached policies =="
        pprint(self.rolePolicies)
        

    def loadRolePolicies(self):
        response = AwsConnectionFactory.getIamClient().list_role_policies(RoleName=self.stackResource.physical_resource_id)

        rolePolicyNames = response['PolicyNames']
        result = {}
        for policyName in rolePolicyNames:
            result[policyName] = AwsConnectionFactory.getIamClient().get_role_policy(RoleName=self.stackResource.physical_resource_id,PolicyName=policyName)

        return result




from AwsProcessor import AwsProcessor
from stdplusAwsHelpers.AwsConnectionFactory import AwsConnectionFactory
from CommandArgumentParser import CommandArgumentParser

from pprint import pprint

class AwsLogStream(AwsProcessor):
    def __init__(self,logStream,parent):
        """Construct an AwsLogStream command processor"""
        AwsProcessor.__init__(self,parent.raw_prompt + "/logStream:" + logStream['logStreamName'],parent)
        self.stackResource = None
        self.logStream = logStream
        self.do_tail([])

    def do_tail(self,args):
        """Tail the logs"""
        response = AwsConnectionFactory.getLogClient().get_log_events(
                logGroupName=self.logStream['logGroupName'],
                logStreamName=self.logStream['logStreamName'],
                limit=10,
                startFromHead=False
            )
        pprint(response)



from AwsProcessor import *
from stdplusAwsHelpers.AwsConnectionFactory import AwsConnectionFactory
from CommandArgumentParser import *
from joblib import Parallel, delayed

import boto3
import stdplus

class AwsAutoScalingGroup(AwsProcessor):
    def __init__(self,scalingGroup,parent):
        AwsProcessor.__init__(self,parent.raw_prompt + "/asg:" + scalingGroup,parent)
        self.client = AwsConnectionFactory.getAsgClient()
        self.scalingGroup = scalingGroup
        self.activities = None
        self.do_printInstances('-r')

    def do_printActivities(self,args):
        """Print scaling activities"""
        parser = CommandArgumentParser("printActivities")
        parser.add_argument('-r','--refresh',action='store_true',dest='refresh',help='refresh');
        args = vars(parser.parse_args(args))
        refresh = args['refresh'] or not self.activities

        if refresh:
            response = self.client.describe_scaling_activities(AutoScalingGroupName=self.scalingGroup)
            self.activities = response['Activities']
        
        index = 0
        for activity in self.activities:
            print "{}: {} -> {} {}: {}".format(index,activity['StartTime'],stdplus.defaultifyDict(activity,'EndTime',''),activity['StatusCode'],activity['Description'])
            index = index + 1

    def do_printActivity(self,args):
        """Print scaling activity details"""
        parser = CommandArgumentParser("printActivity")
        parser.add_argument(dest='index',type=int,help='refresh');
        args = vars(parser.parse_args(args))
        index = args['index']

        activity = self.activities[index]
        pprint(activity)
        
    def do_printInstances(self,args):
        """Print the list of instances in this auto scaling group. printInstances -h for detailed help"""
        parser = CommandArgumentParser("printInstances")
        parser.add_argument(dest='filters',nargs='*',default=["*"],help='Filter instances');
        parser.add_argument('-a','--addresses',action='store_true',dest='addresses',help='list all ip addresses');
        parser.add_argument('-t','--tags',action='store_true',dest='tags',help='list all instance tags');
        parser.add_argument('-d','--allDetails',action='store_true',dest='details',help='print all instance details');
        parser.add_argument('-r','--refresh',action='store_true',dest='refresh',help='refresh');
        args = vars(parser.parse_args(args))
        
        client = AwsConnectionFactory.getEc2Client()

        filters = args['filters']
        addresses = args['addresses']
        tags = args['tags']
        details = args['details']
        needDescription = addresses or tags or details

        if args['refresh']:
            self.scalingGroupDescription = self.client.describe_auto_scaling_groups(AutoScalingGroupNames=[self.scalingGroup])
        
        # print "AutoScaling Group:{}".format(self.scalingGroup)
        print "=== Instances ==="
        instances = self.scalingGroupDescription['AutoScalingGroups'][0]['Instances']

        instances = filter( lambda x: fnmatches(x['InstanceId'],filters),instances)
        
        index = 0
        for instance in instances:
            print "* {0:3d} {1} {2} {3}".format(index,instance['HealthStatus'],instance['AvailabilityZone'],instance['InstanceId'])
            description = None
            if needDescription:
                description = client.describe_instances(InstanceIds=[instance['InstanceId']])
            if addresses:
                networkInterfaces = description['Reservations'][0]['Instances'][0]['NetworkInterfaces']
                number = 0
                print "      Network Interfaces:"
                for interface in networkInterfaces:
                    print "         * {0:3d} {1}".format(number, interface['PrivateIpAddress'])
                    number +=1
            if tags:
                tags = description['Reservations'][0]['Instances'][0]['Tags']
                print "      Tags:"
                for tag in tags:
                    print "        * {0} {1}".format(tag['Key'],tag['Value'])
            if details:
                pprint(description)
                
            index += 1

    def do_printPolicy(self,args):
        """Print the autoscaling policy"""
        parser = CommandArgumentParser("printPolicy")
        args = vars(parser.parse_args(args))

        policy = self.client.describe_policies(AutoScalingGroupName=self.scalingGroup)
        pprint(policy)
        
            
    def do_rebootInstance(self,args):
        """Restart specified instance"""
        parser = CommandArgumentParser("rebootInstance")
        parser.add_argument(dest='instance',help='instance index or name');
        args = vars(parser.parse_args(args))

        instanceId = args['instance']
        try:
            index = int(instanceId)
            instances = self.scalingGroupDescription['AutoScalingGroups'][0]['Instances']
            instanceId = instances[index]
        except ValueError:
            pass

        client = AwsConnectionFactory.getEc2Client()
        client.reboot_instances(InstanceIds=[instanceId['InstanceId']])

        
    def do_setDesiredCapacity(self,args):
        """Set the desired capacity"""
        parser = CommandArgumentParser("setDesiredCapacity")
        parser.add_argument(dest='value',type=int,help='new value');
        args = vars(parser.parse_args(args))

        value = int(args['value'])
        print "Setting desired capacity to {}".format(value)
        client = AwsConnectionFactory.getAsgClient()
        client.set_desired_capacity(AutoScalingGroupName=self.scalingGroup,DesiredCapacity=value,HonorCooldown=True)
        print "Scaling activity in progress"

    def do_run(self,args):
        """SSH to each instance in turn and run specified command"""
        parser = CommandArgumentParser("run")
        parser.add_argument('-R','--replace-key',dest='replaceKey',default=False,action='store_true',help="Replace the host's key. This is useful when AWS recycles an IP address you've seen before.")
        parser.add_argument('-Y','--keyscan',dest='keyscan',default=False,action='store_true',help="Perform a keyscan to avoid having to say 'yes' for a new host. Implies -R.")
        parser.add_argument(dest='command',nargs='+',help="Command to run on all hosts.") # consider adding a filter option later
        parser.add_argument('-v',dest='verbosity',default=0,action=VAction,nargs='?',help='Verbosity. The more instances, the more verbose');        
        parser.add_argument('-j',dest='jobs',type=int,default=1,help='Number of hosts to contact in parallel');
        parser.add_argument('-s',dest='skip',type=int,default=0,help='Skip this many hosts');
        args = vars(parser.parse_args(args))

        replaceKey = args['replaceKey']
        keyscan = args['keyscan']
        verbosity = args['verbosity']
        jobs = args['jobs']
        skip = args['skip']

        instances = self.scalingGroupDescription['AutoScalingGroups'][0]['Instances']
        instances = instances[skip:]
        # if replaceKey or keyscan:
        #     for instance in instances:
        #         stdplus.resetKnownHost(instance)
        
        Parallel(n_jobs=jobs)(
            delayed(ssh)(instance['InstanceId'],0,[],replaceKey,keyscan,False,verbosity," ".join(args['command'])) for instance in instances
        )
        
    def do_ssh(self,args):
        """SSH to an instance. ssh -h for detailed help"""
        parser = CommandArgumentParser("ssh")
        parser.add_argument(dest='instance',help='instance index or name');
        parser.add_argument('-a','--address-number',default='0',dest='interface-number',help='instance id of the instance to ssh to');
        parser.add_argument('-L',dest='forwarding',nargs='*',help="port forwarding string of the form: {localport}:{host-visible-to-instance}:{remoteport} or {port}")
        parser.add_argument('-R','--replace-key',dest='replaceKey',default=False,action='store_true',help="Replace the host's key. This is useful when AWS recycles an IP address you've seen before.")
        parser.add_argument('-Y','--keyscan',dest='keyscan',default=False,action='store_true',help="Perform a keyscan to avoid having to say 'yes' for a new host. Implies -R.")
        parser.add_argument('-B','--background',dest='background',default=False,action='store_true',help="Run in the background. (e.g., forward an ssh session and then do other stuff in aws-shell).")
        parser.add_argument('-v',dest='verbosity',default=0,action=VAction,nargs='?',help='Verbosity. The more instances, the more verbose');        
        args = vars(parser.parse_args(args))

        interfaceNumber = int(args['interface-number'])
        forwarding = args['forwarding']
        replaceKey = args['replaceKey']
        keyscan = args['keyscan']
        background = args['background']
        verbosity = args['verbosity']
        try:
            index = int(args['instance'])
            instances = self.scalingGroupDescription['AutoScalingGroups'][0]['Instances']
            instance = instances[index]
            ssh(instance['InstanceId'],interfaceNumber,forwarding,replaceKey,keyscan,background,verbosity)
        except ValueError:
            ssh(args['instance'],interfaceNumber,forwarding,replaceKey,keyscan,background)

    def do_startInstance(self,args):
        """Start specified instance"""
        parser = CommandArgumentParser("startInstance")
        parser.add_argument(dest='instance',help='instance index or name');
        args = vars(parser.parse_args(args))

        instanceId = args['instance']
        force = args['force']
        try:
            index = int(instanceId)
            instances = self.scalingGroupDescription['AutoScalingGroups'][0]['Instances']
            instanceId = instances[index]
        except ValueError:
            pass

        client = AwsConnectionFactory.getEc2Client()
        client.start_instances(InstanceIds=[instanceId['InstanceId']])
            
    def do_stopInstance(self,args):
        """Stop specified instance"""
        parser = CommandArgumentParser("stopInstance")
        parser.add_argument(dest='instance',help='instance index or name');
        parser.add_argument('-f','--force',action='store_true',dest='force',help='instance index or name');
        args = vars(parser.parse_args(args))

        instanceId = args['instance']
        force = args['force']
        try:
            index = int(instanceId)
            instances = self.scalingGroupDescription['AutoScalingGroups'][0]['Instances']
            instanceId = instances[index]
        except ValueError:
            pass

        client = AwsConnectionFactory.getEc2Client()
        client.stop_instances(InstanceIds=[instanceId['InstanceId']],Force=force)
            
    def do_terminateInstance(self,args):
        """Terminate an EC2 instance"""
        parser = CommandArgumentParser("terminateInstance")
        parser.add_argument(dest='instance',help='instance index or name');
        args = vars(parser.parse_args(args))

        instanceId = args['instance']
        try:
            index = int(instanceId)
            instances = self.scalingGroupDescription['AutoScalingGroups'][0]['Instances']
            instanceId = instances[index]
        except ValueError:
            pass

        client = AwsConnectionFactory.getEc2Client()
        client.terminate_instances(InstanceIds=[instanceId['InstanceId']])
        self.do_printInstances("-r")
            
    def do_updateCapacity(self,args):
        """Set the desired capacity"""
        parser = CommandArgumentParser("updateMinMax")
        parser.add_argument('-m','--min',dest='min',type=int,help='new values');
        parser.add_argument('-M','--max',dest='max',type=int,help='new values');
        parser.add_argument('-d','--desired',dest='desired',type=int,help='desired');
        args = vars(parser.parse_args(args))

        minSize = args['min']
        maxSize = args['max']
        desired = args['desired']
        
        print "Setting desired capacity to {}-{}, {}".format(minSize,maxSize,desired)
        client = AwsConnectionFactory.getAsgClient()
        client.update_auto_scaling_group(AutoScalingGroupName=self.scalingGroup,MinSize=minSize,MaxSize=maxSize,DesiredCapacity=desired)
        #client.set_desired_capacity(AutoScalingGroupName=self.scalingGroup,DesiredCapacity=value,HonorCooldown=True)
        print "Scaling activity in progress"
            



from SilentException import SilentException
from SlashException import SlashException

from stdplusAwsHelpers.AwsConnectionFactory import AwsConnectionFactory
from CommandArgumentParser import *
from stdplus import *

import cmd
import json
import os
import re
import signal
import sys
import traceback
from botocore.exceptions import ClientError
from pprint import pprint

def sshAddress(address,forwarding,replaceKey,keyscan,background,verbosity=0,command=None): 
    if replaceKey or keyscan:
        resetKnownHost(address)

    if keyscan:
        keyscanHost(address)

    args=["/usr/bin/ssh",address]
    if not forwarding == None:
        for forwardInfo in forwarding:
            if isInt(forwardInfo):
                forwardInfo = "{0}:localhost:{0}".format(forwardInfo)
            args.extend(["-L",forwardInfo])
        if background:
            args.extend(["-N","-n"])
    else:
        background = False # Background is ignored if not forwarding

    if verbosity > 0:
        args.append("-" + "v" * verbosity)

    if command:
        args.append(command)

    print " ".join(args)
    pid = fexecvp(args)
    if background:
        print "SSH Started in background. pid:{}".format(pid)
        AwsProcessor.backgroundTasks.append(pid)
    else:
        os.waitpid(pid,0)
   

def ssh(instanceId,interfaceNumber,forwarding,replaceKey,keyscan,background,verbosity=0,command=None):
    if isIp(instanceId):
        sshAddress(instanceId,forwarding,replaceKey,keyscan,background,verbosity,command)
    else:
        client = AwsConnectionFactory.getEc2Client()
        response = client.describe_instances(InstanceIds=[instanceId])
        networkInterfaces = response['Reservations'][0]['Instances'][0]['NetworkInterfaces'];
        if None == interfaceNumber:
            number = 0
            for interface in networkInterfaces:
                print "{0:3d} {1}".format(number,interface['PrivateIpAddress'])
                number += 1
        else:
            address = "{}".format(networkInterfaces[interfaceNumber]['PrivateIpAddress'])
            sshAddress(address,forwarding,replaceKey,keyscan,background,verbosity,command)

class AwsProcessor(cmd.Cmd):
    backgroundTasks=[]
    resourceTypeAliases={ 'AWS::AutoScaling::AutoScalingGroup' : 'asg',
                          'AWS::CloudFormation::Stack' : 'stack',
                          'AWS::EC2::NetworkInterface' : 'eni',
                          'AWS::Logs::LogGroup' : 'logGroup' }
    processorFactory = None
    
    def __init__(self,prompt,parent):
        cmd.Cmd.__init__(self)
        self.raw_prompt = prompt
        self.prompt = prompt + "/: "
        self.parent = parent

    def emptyline(self):
        pass

    @staticmethod
    def killBackgroundTasks():
        for pid in AwsProcessor.backgroundTasks:
            print "Killing pid:{}".format(pid)
            os.kill(pid,signal.SIGQUIT)
    
    def onecmd(self, line):
        try:
            return cmd.Cmd.onecmd(self,line)
        except SystemExit, e:
            raise e;
        except SlashException, e:
            if None == self.parent:
                pass
            else:
                raise e
        except SilentException:
            pass
        except ClientError as e:
            # traceback.print_exc()
            if e.response['Error']['Code'] == 'AccessDenied':
                print "ERROR: Access Denied. Maybe you need to run mfa {code}"
                traceback.print_exc()
        except Exception, other:
            traceback.print_exc()
        except:
            print "Unexpected error:", sys.exc_info()[0]

    def mfa_devices(self, profile='default'):
        list_mfa_devices_command = ["aws","--profile",profile,"--output","json","iam","list-mfa-devices"]
        result = run_cmd(list_mfa_devices_command)
        if result.retCode == 0 :
            return json.loads("\n".join(result.stdout))['MFADevices']
        else:
            raise Exception('There was a problem fetching MFA devices from AWS')
            
    def load_arn_from_aws(self, profile):
        devices = self.mfa_devices(profile)
        if len(devices):
            return devices[0]['SerialNumber']
        else:
            raise Exception('No MFA devices were found for your account')

    def do_mfa(self, args):
        """Enter a 6-digit MFA token. mfa -h for more details"""
        parser = CommandArgumentParser("mfa")
        parser.add_argument(dest='token',help='MFA token value');
        parser.add_argument("-p","--profile",dest='profile',default=AwsConnectionFactory.instance.getProfile(),help='MFA token value');
        args = vars(parser.parse_args(args))

        token = args['token']
        profile = args['profile']
        arn = AwsConnectionFactory.instance.load_arn(profile)

        credentials_command = ["aws","--profile",profile,"--output","json","sts","get-session-token","--serial-number",arn,"--token-code",token]
        output = run_cmd(credentials_command) # Throws on non-zero exit :yey:

        credentials = json.loads("\n".join(output.stdout))['Credentials']
        AwsConnectionFactory.instance.setMfaCredentials(credentials,profile)

    def do_up(self,args):
        """Go up one level"""
        if None == self.parent:
            print "You're at the root. Try 'quit' to quit"
        else:
            return True

    def do_slash(self,args):
        """Go up to root level"""
        if None == self.parent:
            print "You're at the root. Try 'quit' to quit"
        else:
            raise SlashException()

    def do_profile(self,args):
        """Select AWS profile"""
        parser = CommandArgumentParser("profile")
        parser.add_argument(dest="profile",help="Profile name")
        parser.add_argument('-v','--verbose',dest="verbose",action='store_true',help='verbose')
        args = vars(parser.parse_args(args))

        profile = args['profile']
        verbose = args['verbose']
        if verbose:
            print "Selecting profile '{}'".format(profile)
        AwsConnectionFactory.resetInstance(profile=profile)

    def do_quit(self,args):
        """Alias for 'exit'"""
        return self.do_exit(args)
        
    def do_exit(self,args):
        """Exit cf-ui"""
        raise SystemExit

    def childLoop(self,child):
        try:
            child.cmdloop()
        except SilentException, e:
            raise e
        except SlashException, e:
            raise e
        except Exception, e:
            print "Exception: {}".format(e)
            traceback.print_exc()

    def stackResource(self,stackName,logicalId):
        print "loading stack resource {}.{}".format(stackName,logicalId)
        stackResource = AwsConnectionFactory.instance.getCfResource().StackResource(stackName,logicalId)
        pprint(stackResource)
        if "AWS::CloudFormation::Stack" == stackResource.resource_type:
            pprint(stackResource)
            print "Found a stack w/ physical id:{}".format(stackResource.physical_resource_id)
            childStack = AwsConnectionFactory.instance.getCfResource().Stack(stackResource.physical_resource_id)
            print "Creating prompt"
            self.childLoop(AwsProcessor.processorFactory.Stack(childStack,logicalId,self))
        elif "AWS::AutoScaling::AutoScalingGroup" == stackResource.resource_type:
            scalingGroup = stackResource.physical_resource_id
            self.childLoop(AwsProcessor.processorFactory.AutoScalingGroup(scalingGroup,self))
        elif "AWS::EC2::NetworkInterface" == stackResource.resource_type:
            eniId = stackResource.physical_resource_id
            self.childLoop(AwsProcessor.processorFactory.Eni(eniId,self))
        elif "AWS::Logs::LogGroup" == stackResource.resource_type:
            self.childLoop(AwsProcessor.processorFactory.LogGroup(stackResource,self))
        elif "AWS::IAM::Role" == stackResource.resource_type:
            self.childLoop(AwsProcessor.processorFactory.Role(stackResource,self))
        else:
            pprint(stackResource)
            print("- description:{}".format(stackResource.description))
            print("- last_updated_timestamp:{}".format(stackResource.last_updated_timestamp))
            print("- logical_resource_id:{}".format(stackResource.logical_resource_id))
            print("- metadata:{}".format(stackResource.metadata.strip()))
            print("- physical_resource_id:{}".format(stackResource.physical_resource_id))
            print("- resource_status:{}".format(stackResource.resource_status))
            print("- resource_status_reason:{}".format(stackResource.resource_status_reason))
            print("- resource_type:{}".format(stackResource.resource_type))
            print("- stack_id:{}".format(stackResource.stack_id))

    def do_ssh(self,args):
        """SSH to an instance. ssh -h for detailed help."""
        parser = CommandArgumentParser()
        parser.add_argument(dest='id',help='identifier of the instance to ssh to [aws instance-id or ip address]')
        parser.add_argument('-a','--interface-number',dest='interface-number',default='0',help='instance id of the instance to ssh to')
        parser.add_argument('-L',dest='forwarding',nargs='*',help="port forwarding string: {localport}:{host-visible-to-instance}:{remoteport} or {port}")
        parser.add_argument('-R','--replace-key',dest='replaceKey',default=False,action='store_true',help="Replace the host's key. This is useful when AWS recycles an IP address you've seen before.")
        parser.add_argument('-Y','--keyscan',dest='keyscan',default=False,action='store_true',help="Perform a keyscan to avoid having to say 'yes' for a new host. Implies -R.")
        parser.add_argument('-B','--background',dest='background',default=False,action='store_true',help="Run in the background. (e.g., forward an ssh session and then do other stuff in aws-shell).")
        parser.add_argument('-v',dest='verbosity',default=0,action=VAction,nargs='?',help='Verbosity. The more instances, the more verbose');
        args = vars(parser.parse_args(args))

        targetId = args['id']
        interfaceNumber = int(args['interface-number'])
        forwarding = args['forwarding']
        replaceKey = args['replaceKey']
        keyscan = args['keyscan']
        background = args['background']
        verbosity = args['verbosity']
        ssh(targetId,interfaceNumber, forwarding, replaceKey, keyscan, background, verbosity)




#!/usr/local/bin/python

import argparse
import atexit
import boto3
import cmd

import json
import os
import readline
import shlex
import subprocess
import sys
import traceback
import yaml

from stdplusAwsHelpers.AwsConnectionFactory import AwsConnectionFactory
from AwsProcessor import AwsProcessor
from AwsProcessorFactoryImpl import AwsProcessorFactoryImpl
from CommandArgumentParser import CommandArgumentParser        

from botocore.exceptions import ClientError
from fnmatch import fnmatch
from pprint import pprint
from stdplus import *

stackStatusFilter=['CREATE_COMPLETE','CREATE_FAILED','CREATE_IN_PROGRESS','ROLLBACK_IN_PROGRESS','ROLLBACK_COMPLETE','UPDATE_COMPLETE','DELETE_IN_PROGRESS']

mappedKeys = { 'SecretAccessKey' : 'AWS_SECRET_ACCESS_KEY', 'SessionToken': 'AWS_SECURITY_TOKEN', 'AccessKeyId' : 'AWS_ACCESS_KEY_ID' }

from SilentException import SilentException
from SlashException import SlashException

from AwsAutoScalingGroup import AwsAutoScalingGroup        
from AwsStack import AwsStack
            
class AwsRoot(AwsProcessor):
    def __init__(self):
        AwsProcessor.__init__(self,"(aws)",None)
        self.stackList = None;

    def do_stack(self,args):
        """Go to the specified stack. stack -h for detailed help"""
        parser = CommandArgumentParser("stack")
        parser.add_argument(dest='stack',help='stack index or name');
        args = vars(parser.parse_args(args))

        try:
            index = int(args['stack'])
            if self.stackList == None:
                self.do_stacks('-s')
            stack = AwsConnectionFactory.instance.getCfResource().Stack(self.stackList[index]['StackName'])
        except ValueError:
            stack = AwsConnectionFactory.instance.getCfResource().Stack(args['stack'])

        AwsProcessor.processorFactory.Stack(stack,stack.name,self).cmdloop()

    def do_delete_stack(self,args):
        """Delete specified stack. delete_stack -h for detailed help."""
        parser = CommandArgumentParser("delete_stack")
        parser.add_argument(dest='stack',help='stack index or name');
        args = vars(parser.parse_args(args))

        try:
            index = int(args['stack'])
            if self.stackList == None:
                self.do_stacks('-s')
            stack = AwsConnectionFactory.instance.getCfResource().Stack(self.stackList[index]['StackName'])
        except ValueError:
            stack = AwsConnectionFactory.instance.getCfResource().Stack(args['stack'])

        print "Here are the details of the stack you are about to delete:"
        print "Stack.name: {}".format(stack.name)
        print "Stack.stack_id: {}".format(stack.stack_id)
        print "Stack.creation_time: {}".format(stack.creation_time)
        confirmation = raw_input("If you are sure, enter the Stack.name here: ")
        if stack.name == confirmation:
            stack.delete()
            print "Stack deletion in progress"
        else:
            print "Stack deletion canceled: '{}' != '{}'".format(stack.name,confirmation)

    def do_stacks(self,args):
        """List available stacks. stacks -h for detailed help."""
        parser = CommandArgumentParser()
        parser.add_argument('-s','--silent',dest='silent',action='store_true',help='Run silently');
        parser.add_argument('-i','--include',nargs='*',dest='includes',default=[],help='Add statuses');
        parser.add_argument('-e','--exclude',nargs='*',dest='excludes',default=[],help='Remove statuses');
        parser.add_argument(dest='filters',nargs='*',default=["*"],help='Filter stacks');
        args = vars(parser.parse_args(args))

        nextToken = None

        includes = args['includes']
        excludes = args['excludes']
        filters = args['filters']

        global stackStatusFilter
        for i in includes:            
            if not i in stackStatusFilter:
                stackStatusFilter.append(i)
        for e in excludes:
            stackStatusFilter.remove(e)

        complete = False;
        stackSummaries = []
        while not complete:
            if None == nextToken:
                stacks = AwsConnectionFactory.getCfClient().list_stacks(StackStatusFilter=stackStatusFilter)
            else:
                stacks = AwsConnectionFactory.getCfClient().list_stacks(NextToken=nextToken,StackStatusFilter=stackStatusFilter)
                #pprint(stacks)
            if not 'NextToken' in stacks:
                complete = True;
            else:
                nextToken = stacks['NextToken']

            if 'StackSummaries' in stacks:
                stackSummaries.extend(stacks['StackSummaries'])

        stackSummaries = filter( lambda x: fnmatches(x['StackName'],filters),stackSummaries)
        stackSummaries = sorted(stackSummaries, key= lambda entry: entry['StackName'])
        index = 0;
        stackSummariesByIndex = {}
        for summary in stackSummaries:
            summary['Index'] = index
            stackSummariesByIndex[index] = summary
            index += 1

        self.stackList = stackSummariesByIndex
        if not args['silent']:
            for index,summary in stackSummariesByIndex.items():
                print '{0:3d}: {2:20} {1:40} {3}'.format(summary['Index'],summary['StackName'],summary['StackStatus'],defaultifyDict(summary,'StackStatusReason',''))
        
    def do_stack_resource(self, args):
        """Use specified stack resource. stack_resource -h for detailed help."""
        parser = CommandArgumentParser()
        parser.add_argument('-s','--stack-name',dest='stack-name',help='name of the stack resource');
        parser.add_argument('-i','--logical-id',dest='logical-id',help='logical id of the child resource');
        args = vars(parser.parse_args(args))

        stackName = args['stack-name']
        logicalId = args['logical-id']

        self.stackResource(stackName,logicalId)

def main():
    try:
        argv = sys.argv
        configFile = os.path.join(os.path.expanduser("~"),".aws-shell.yaml")
        config={}
        if os.path.exists(configFile):
            print "Loading config:{}".format(configFile)
            config = yaml.load(readfile(configFile))        

        parser = CommandArgumentParser(argv[0])
        parser.add_argument('-p','--profile',dest='profile',default=defaultifyDict(config,'profile','default'),help='select aws profile');
        parser.add_argument('-m','--mfa',dest='mfa',help='provide mfa code');
        args = vars(parser.parse_args(argv[1:]))

        histfile = os.path.join(os.path.expanduser("~"), ".aws_hist")
        try:
            readline.read_history_file(histfile)
            readline.set_history_length(1000)
        except IOError:
            pass
        atexit.register(readline.write_history_file, histfile)

        atexit.register(AwsProcessor.killBackgroundTasks)
        AwsProcessor.processorFactory = AwsProcessorFactoryImpl()

        awsConfigFilename = os.path.expanduser("~/.aws/config")
        if not os.path.exists(awsConfigFilename):
            print "ERROR: aws cli has not been configured."
            pid = fexecvp(['aws','configure'])
            os.waitpid(pid,0)

        command_prompt = AwsRoot()
        command_prompt.onecmd("profile -v {}".format(args['profile']))
        if None != args['mfa']:
            command_prompt.onecmd("mfa {}".format(args['mfa']))

        command_prompt.onecmd("stacks")
        command_prompt.cmdloop()
    except SilentException:
        pass

if __name__== "__main__":
    main()
        



import argparse
import cmd
import shlex

from SilentException import SilentException

class VAction(argparse.Action):
    def __call__(self, parser, args, values, option_string=None):
        # print 'values: {v!r}'.format(v=values)
        if values==None:
            values='1'
        try:
            values=int(values)
        except ValueError:
            values=values.count('v')+1
        setattr(args, self.dest, values)

class CommandArgumentParser(argparse.ArgumentParser):
    def __init__(self,command = None):
        argparse.ArgumentParser.__init__(self, prog=command)
        
    def exit(self, status=0, message=None):
        if None == message:
            raise SilentException()
        else:
            raise Exception(message)

    def error(self, message):
        raise Exception(message)

    def parse_args(self,commandLine):
        if isinstance(commandLine, basestring):
            commandLine = shlex.split(commandLine)
        return super(CommandArgumentParser,self).parse_args(commandLine)
    



import AwsAutoScalingGroup
import AwsLogGroup
import AwsRole
import AwsStack
import AwsEni

class AwsProcessorFactoryImpl:
    def AutoScalingGroup(self,scalingGroup,parent):
        reload(AwsAutoScalingGroup)
        return AwsAutoScalingGroup.AwsAutoScalingGroup(scalingGroup,parent)

    def Eni(self,physicalId,parent):
        reload(AwsEni)
        return AwsEni.AwsEni(physicalId,parent)
    
    def Stack(self,stack,logicalName,parent):
        reload(AwsStack)
        return AwsStack.AwsStack(stack,logicalName,parent)
    
    def LogGroup(self,logGroupId,parent):
        reload(AwsLogGroup)
        return AwsLogGroup.AwsLogGroup(logGroupId,parent)

    def Role(self,roleId,parent):
        reload(AwsRole)
        return AwsRole.AwsRole(roleId,parent)
